@@ -27,6 +27,7 @@
 #include "xrdp_rail.h"
 #include "xrdp_constants.h"
 #include "fifo.h"
+#include "guid.h"
 
 #define MAX_NR_CHANNELS 16
 #define MAX_CHANNEL_NAME 16
@@ -72,7 +73,7 @@ struct xrdp_mod
     int (*server_set_pointer)(struct xrdp_mod *v, int x, int y,
                               char *data, char *mask);
     int (*server_palette)(struct xrdp_mod *v, int *palette);
-    int (*server_msg)(struct xrdp_mod *v, char *msg, int code);
+    int (*server_msg)(struct xrdp_mod *v, const char *msg, int code);
     int (*server_is_term)(struct xrdp_mod *v);
     int (*server_set_clip)(struct xrdp_mod *v, int x, int y, int cx, int cy);
     int (*server_reset_clip)(struct xrdp_mod *v);
@@ -95,6 +96,7 @@ struct xrdp_mod
                             int box_right, int box_bottom,
                             int x, int y, char *data, int data_len);
     int (*server_reset)(struct xrdp_mod *v, int width, int height, int bpp);
+    int (*server_get_channel_count)(struct xrdp_mod *v);
     int (*server_query_channel)(struct xrdp_mod *v, int index,
                                 char *channel_name,
                                 int *channel_flags);
@@ -159,7 +161,7 @@ struct xrdp_mod
                               int flags, int frame_id);
     int (*server_session_info)(struct xrdp_mod *v, const char *data,
                                int data_bytes);
-    tintptr server_dumby[100 - 45]; /* align, 100 minus the number of server
+    tintptr server_dumby[100 - 46]; /* align, 100 minus the number of server
                                      functions above */
     /* common */
     tintptr handle; /* pointer to self as int */
@@ -168,6 +170,16 @@ struct xrdp_mod
     struct source_info *si;
 };
 
+/**
+ * Transform to apply to loaded images
+ */
+enum xrdp_bitmap_load_transform
+{
+    XBLT_NONE = 0,
+    XBLT_SCALE,
+    XBLT_ZOOM
+};
+
 /* header for bmp file */
 struct xrdp_bmp_header
 {
@@ -284,13 +296,40 @@ struct xrdp_cache
 /* defined later */
 struct xrdp_enc_data;
 
+/**
+ * Stages we go through connecting to the session
+ */
+enum mm_connect_state
+{
+    MMCS_CONNECT_TO_SESMAN,
+    MMCS_PAM_AUTH,
+    MMCS_SESSION_AUTH,
+    MMCS_CONNECT_TO_SESSION,
+    MMCS_CONNECT_TO_CHANSRV,
+    MMCS_DONE
+};
+
 struct xrdp_mm
 {
     struct xrdp_wm *wm; /* owner */
-    int connected_state; /* true if connected to sesman else false */
+    enum mm_connect_state connect_state; /* State of connection */
+    /* Other processes we connect to */
+    /* NB : When we move to UDS, the sesman and pam_auth
+     * connection be merged */
+    int use_sesman; /* true if this is a sesman session */
+    int use_pam_auth; /* True if we're to authenticate using PAM */
+    int use_chansrv; /* true if chansrvport is set in xrdp.ini or using sesman */
     struct trans *sesman_trans; /* connection to sesman */
-    int sesman_trans_up; /* true once connected to sesman */
-    int delete_sesman_trans; /* boolean set when done with sesman connection */
+    struct trans *pam_auth_trans; /* connection to pam authenticator */
+    struct trans *chan_trans; /* connection to chansrv */
+
+    /* We can't delete transports while we're in a callback for that
+     * transport, as this causes trans.c to reference undefined memory.
+     * These flags mark transports as needing to be deleted when
+     * we are definitely not in a transport callback */
+    int delete_sesman_trans;
+    int delete_pam_auth_trans;
+
     struct list *login_names;
     struct list *login_values;
     /* mod vars */
@@ -299,12 +338,8 @@ struct xrdp_mm
     int (*mod_exit)(struct xrdp_mod *);
     struct xrdp_mod *mod; /* module interface */
     int display; /* 10 for :10.0, 11 for :11.0, etc */
+    struct guid guid; /* GUID for the session, or all zeros  */
     int code; /* 0=Xvnc session, 10=X11rdp session, 20=xorg driver mode */
-    int sesman_controlled; /* true if this is a sesman session */
-    struct trans *chan_trans; /* connection to chansrv */
-    int chan_trans_up; /* true once connected to chansrv */
-    int delete_chan_trans; /* boolean set when done with channel connection */
-    int usechansrv; /* true if chansrvport is set in xrdp.ini or using sesman */
     struct xrdp_encoder *encoder;
     int cs2xr_cid_map[256];
     int xr2cr_cid_map[256];
@@ -634,8 +669,14 @@ struct xrdp_cfg_globals
     int  ls_width;               /* window width */
     int  ls_height;              /* window height */
     int  ls_bg_color;            /* background color */
-    char ls_logo_filename[256];  /* logo filename */
     char ls_background_image[256];  /* background image file name */
+    enum xrdp_bitmap_load_transform ls_background_transform;
+    /* transform to apply to background image */
+    char ls_logo_filename[256];  /* logo filename */
+    enum xrdp_bitmap_load_transform ls_logo_transform;
+    /* transform to apply to logo */
+    int  ls_logo_width;          /* logo width (optional) */
+    int  ls_logo_height;          /* logo height (optional) */
     int  ls_logo_x_pos;          /* logo x co-ordinate */
     int  ls_logo_y_pos;          /* logo y co-ordinate */
     int  ls_label_x_pos;         /* x pos of labels */
